// exfleurdat.cpp, v1.0 2013/12/28
// coded by asmodean

// contact: 
//   web:   http://asmodean.reverse.net
//   email: asmodean [at] hush.com
//   irc:   asmodean on efnet (irc.efnet.net)

// This tool extracts *.dat archives used in VX^[.

#include "as-util.h"
#include "as-lzss.h"

struct DATHDR {
  uint32_t entry_count;
  uint32_t unknown1;
  uint32_t unknown2;
  uint32_t unknown3;
  uint32_t unknown4;
  uint32_t unknown5;
  uint32_t unknown6;
  uint32_t unknown7;
};

struct DATENTRY {
  char     filename[256];
  uint32_t original_length;
  uint32_t offset;
  uint32_t unknown1;
  uint32_t unknown2;
};

struct PRSHDR {
  uint8_t  signature[16]; // "PRS06X"
  uint32_t original_length;
  uint32_t length;
  uint32_t unknown1;
  uint32_t unknown2;
  uint32_t unknown3;
};

int main(int argc, char** argv) {
  if (argc != 2) {
    fprintf(stderr, "exfleurdat v1.0, coded by asmodean\n\n");
    fprintf(stderr, "usage: %s <input.dat>\n", argv[0]);
    return -1;
  }

  string in_filename = argv[1];

  int fd = as::open_or_die(in_filename, O_RDONLY | O_BINARY); 

  DATHDR hdr;
  read(fd, &hdr, sizeof(hdr));

  auto entries = new DATENTRY[hdr.entry_count];
  read(fd, entries, sizeof(DATENTRY) * hdr.entry_count);

  for (uint32_t i = 0; i < hdr.entry_count; i++) {
    uint32_t len  = (i + 1 == hdr.entry_count ? as::get_file_size(fd) : entries[i + 1].offset) - entries[i].offset;
    auto     buff = new uint8_t[len];
    lseek(fd, entries[i].offset, SEEK_SET);
    read(fd, buff, len);

    if (len >= sizeof(PRSHDR) && !memcmp(buff, "PRS", 3)) {
      auto prshdr = (PRSHDR*) buff;
      auto data   = (uint8_t*) (prshdr + 1);

      uint32_t temp_len  = prshdr->original_length;
      auto     temp_buff = new uint8_t[temp_len];

      if (prshdr->length != prshdr->original_length) {
        as::unlzss(data, prshdr->length, temp_buff, temp_len);
      } else {
        memcpy(temp_buff, data, prshdr->original_length);
      }

      delete [] buff;

      len  = temp_len;
      buff = temp_buff;
    }

    as::write_file(entries[i].filename, buff, len);

    delete [] buff;
  }

  delete [] entries;

  close(fd);

  return 0;
}
